//////////////////////////////////////////////
// Memory Game - (c) 2008 Burkhard Wuensche //
//////////////////////////////////////////////
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>

#include <time.h>
#include <iostream>
using namespace std;
#include <stdlib.h>
#include <time.h>
#include "MyMath.h"
#include "Object2df.h"
#include "Card.h"

//#define ELLIPSOID_IMPLEMENTED

typedef enum {NO_CARD_SHOWN, ONE_CARD_SHOWN, TWO_CARDS_SHOWN} GameStateType;
GameStateType gameState;
int firstSelectedCard, secondSelectedCard;

const int windowWidth=1000;
const int windowHeight=800;

const int numCards=20;
int numMoves;
int numFoundPairs;

CCard cards[20];			// an array of cards
CObject2df* shapePtr[10];	// an array of shapes used for drawing the cards

clock_t currentSystemTime, previousSystemTime, startSystemTime;
double timeGame, timeAnimationStep, maxTime=90;

void display(void)
{
	// clear all pixels in frame buffer
	glClear(GL_COLOR_BUFFER_BIT);

	// draw the array of objects
	for(int i=0;i<numCards;i++) cards[i].draw();

	// start processing buffered OpenGL routines
	glFlush ();
	glutSwapBuffers();
}


void init(void)
{
	// select clearing color (for glClear)
	glClearColor (1.0, 1.0, 1.0, 0.0);	// RGB-value for white

	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0,windowWidth,0,windowHeight);
}

CObject2df* createEllipsoid(int r, int g, int b, float majorRadius, float minorRadius){
#ifdef ELLIPSOID_IMPLEMENTED
	CObject2df* objectPtr=new CEllipsoid2df(CVec2df(0,0),majorRadius,minorRadius);
	objectPtr->setColour(r,g,b);
	return objectPtr;
#else
	return NULL;
#endif
}

CObject2df* createTriangle(int r, int g, int b, float sideLengthHalf){
	CVec2df *vertices=new CVec2df[3];
	vertices[0].set(-sideLengthHalf,-sideLengthHalf); vertices[1].set(-sideLengthHalf,sideLengthHalf); vertices[2].set(sideLengthHalf,0);
	CObject2df* objectPtr=new CConvexPolygon2df(3, vertices);
	objectPtr->setColour(r,g,b);
	return objectPtr;
}

CObject2df* createSquare(int r, int g, int b, float sideLengthHalf){
	CVec2df *vertices=new CVec2df[4];
	vertices[0].set(-sideLengthHalf,-sideLengthHalf); vertices[1].set(sideLengthHalf,-sideLengthHalf); 
	vertices[2].set(sideLengthHalf,sideLengthHalf); vertices[3].set(-sideLengthHalf,sideLengthHalf);
	CObject2df* objectPtr=new CConvexPolygon2df(4, vertices);
	objectPtr->setColour(r,g,b);
	return objectPtr;
}

void initCards(){
	// create array of 20 integers (indicating positions on 5x4 game board) and shuffle it
	int i, index, temp;

	int p[20];
	for(i=0;i<20;i++) p[i]=i;
	srand ( time(NULL) );	// initialize random seed
	for(i=0;i<20;i++) { index=rand()%20; temp=p[index]; p[index]=p[i]; p[i]=temp;}

	// create 10 pairs of cards - the cards' positions on a 5x4 pattern are given by p[i]
#ifdef ELLIPSOID_IMPLEMENTED
	shapePtr[0]=createEllipsoid(255,0,0, 20.0,20.0);
#else
	shapePtr[0]=createTriangle(255,0,0, 25.0);
#endif
	cards[0].init(p[0],shapePtr[0]);
	cards[1].init(p[1],shapePtr[0]);

#ifdef ELLIPSOID_IMPLEMENTED
	shapePtr[1]=createEllipsoid(0,255,0, 20.0,20.0);
#else
	shapePtr[1]=createTriangle(0,255,0, 25.0);
#endif
	cards[2].init(p[2],shapePtr[1]);
	cards[3].init(p[3],shapePtr[1]);

#ifdef ELLIPSOID_IMPLEMENTED
	shapePtr[2]=createEllipsoid(0,0,255, 20.0,20.0);
#else
	shapePtr[2]=createTriangle(0,0,255, 25.0);
#endif
	cards[4].init(p[4],shapePtr[2]);
	cards[5].init(p[5],shapePtr[2]);

#ifdef ELLIPSOID_IMPLEMENTED
	shapePtr[3]=createEllipsoid(255,255,0, 20.0,20.0);
#else
	shapePtr[3]=createTriangle(255,255,0, 25.0);
#endif
	cards[6].init(p[6],shapePtr[3]);
	cards[7].init(p[7],shapePtr[3]);

	shapePtr[4]=createTriangle(255,0,0, 15.0);
	cards[8].init(p[8],shapePtr[4]);
	cards[9].init(p[9],shapePtr[4]);

	shapePtr[5]=createTriangle(0,255,0, 15.0);
	cards[10].init(p[10],shapePtr[5]);
	cards[11].init(p[11],shapePtr[5]);

	shapePtr[6]=createTriangle(0,0,255, 15.0);
	cards[12].init(p[12],shapePtr[6]);
	cards[13].init(p[13],shapePtr[6]);

	shapePtr[7]=createSquare(255,0,0, 25.0);
	cards[14].init(p[14],shapePtr[7]);
	cards[15].init(p[15],shapePtr[7]);

	shapePtr[8]=createSquare(0,255,0, 25.0);
	cards[16].init(p[16],shapePtr[8]);
	cards[17].init(p[17],shapePtr[8]);

	shapePtr[9]=createSquare(0,0,255, 25.0);
	cards[18].init(p[18],shapePtr[9]);
	cards[19].init(p[19],shapePtr[9]);
}


void geometryLibraryTest(){
	CVec2df v1;
	v1[X]=3.0f; v1[Y]=1.0f;
	CVec2df v2(1.0, 2.0);
	v2*=5.0f;
	CVec2df v=v1+v2;
	cout << "\n" << "v1+v2" << "=" << "(" << v[X] << ", " << v[Y] << ")";
//	float dot=v1*v2;
//	cout << "\n" << v1 << "*" << v2 << "=" << dot << "\n";
}

void startGame(){
	initCards();
	startSystemTime = previousSystemTime = clock();
	gameState=NO_CARD_SHOWN;
	firstSelectedCard=secondSelectedCard=-1;
	numMoves=0;
	numFoundPairs=0;
	cout << "\nSTART GAME";
}

void endGame(){
	if (timeGame>maxTime){
		cout << "\n GAME OVER - MAXIMUM TIME EXCEEDED";
		cout << "\n You found " << numFoundPairs << " pairs!";
	}
	if (numFoundPairs==10){
		cout << "\n GAME OVER - ALL PAIRS FOUND";
		cout << "\n You found all pairs in " << timeGame << " seconds!";
	}
	cout << "\n You required " << numMoves << " moves!";
	cout << "\n Total score: " << (int) ((maxTime-timeGame)*(float) 100/(float) numMoves) << " points!";
}

void handleKeyPress(unsigned char key, int x, int y)
{
}



void animate(void)
{
	currentSystemTime = clock();
	timeAnimationStep = ((double)(currentSystemTime - previousSystemTime))/((double) CLOCKS_PER_SEC);
	previousSystemTime=currentSystemTime;
	if (timeAnimationStep>0.002){
		timeGame = ((double)(currentSystemTime - startSystemTime))/((double) CLOCKS_PER_SEC);
		if (timeGame>maxTime) endGame();
	}
}

void handleMouseClick(int button, int state, int x, int y)
{
	if (state == GLUT_DOWN){ // only do event handling if mouse button pressed (i.e. not at release)
		if (numMoves==0 && NO_CARD_SHOWN) startGame();	// start game withthe first mouse click
		if (gameState==TWO_CARDS_SHOWN){		
			if (cards[secondSelectedCard].sameCard(cards[firstSelectedCard])){
				cards[secondSelectedCard].cardState=INVISIBLE;
				cards[firstSelectedCard].cardState=INVISIBLE;
			} else {
				cards[secondSelectedCard].cardState=SHOW_BACK;
				cards[firstSelectedCard].cardState=SHOW_BACK;
			}
			gameState=NO_CARD_SHOWN;
			glutPostRedisplay();
			firstSelectedCard=secondSelectedCard=-1;
		}
		for(int i=0;i<numCards;i++) 
			if (cards[i].cardState!=INVISIBLE && cards[i].isSelected(x,windowHeight-y)){
				switch(gameState){
					case NO_CARD_SHOWN:
						if (cards[i].cardState==SHOW_BACK) cards[i].cardState=SHOW_FRONT;
						firstSelectedCard=i;
						numMoves++;
						gameState=ONE_CARD_SHOWN;
						break;
					case ONE_CARD_SHOWN:
						if (i==firstSelectedCard) break;
						if (cards[i].cardState==SHOW_BACK){
							secondSelectedCard=i;
							cards[i].cardState=SHOW_FRONT;
							if (cards[i].sameCard(cards[firstSelectedCard])){
								cout << "Pair Found!";
								numFoundPairs++;
								if (numFoundPairs==10) endGame();
							}
							gameState=TWO_CARDS_SHOWN;
						}
						break;
				}
				glutPostRedisplay();
			}
	}	
}

// create a single buffered big colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(windowWidth, windowHeight);
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Assignment 1, Question 3");
	glutDisplayFunc(display);		// draw scene
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutKeyboardFunc(handleKeyPress);		// Set function to handle pressing of key
	init();							// initialise view

	// Example of using the class CVec3df
	geometryLibraryTest();

	initCards();
	glutIdleFunc(animate);
	glutMainLoop();
	return 0;
}
